@@ -3,10 +3,14 @@
 # by Steven Hazel
 
 import urlparse
-import ordereddict
 
-from monocle import _o, VERSION
-from monocle.deferred import Deferred
+try:
+    from collections import OrderedDict
+except ImportError:
+    from ordereddict import OrderedDict
+
+from monocle import _o, Return, VERSION, launch
+from monocle.callback import Callback
 from monocle.twisted_stack.eventloop import reactor
 
 from twisted.internet import ssl
@@ -17,7 +21,7 @@ from twisted.web import server, resource
 
 class HttpException(Exception): pass
 
-class HttpHeaders(ordereddict.OrderedDict): pass
+class HttpHeaders(OrderedDict): pass
 
 
 class HttpResponse(object):
@@ -34,12 +38,12 @@ class _HttpClient(TwistedHTTPClient):
         except AttributeError:
             pass
         self.code = None
-        self.headers = ordereddict.OrderedDict()
-        self.connect_df = Deferred()
-        self.response_df = Deferred()
+        self.headers = OrderedDict()
+        self.connect_cb = Callback()
+        self.response_cb = Callback()
 
     def connectionMade(self):
-        self.connect_df.callback(None)
+        self.connect_cb(None)
 
     def handleStatus(self, protocol, code, message):
         self.code = code
@@ -48,7 +52,7 @@ class _HttpClient(TwistedHTTPClient):
         self.headers[name] = value
 
     def handleResponse(self, data):
-        self.response_df.callback(HttpResponse(self.code, self.headers, data))
+        self.response_cb(HttpResponse(self.code, self.headers, data))
 
     def close(self):
         if self.transport:
@@ -68,7 +72,7 @@ class HttpClient(object):
         return self._HEADER_NORMS.get(name.lower(), name)
 
     def _normalize_headers(self, headers):
-        return ordereddict.OrderedDict(
+        return OrderedDict(
             ((self._normalize_header_name(key), value)
              for key, value in headers.iteritems()))
 
@@ -84,7 +88,7 @@ class HttpClient(object):
                                              ssl.ClientContextFactory())
         else:
             raise HttpException('unsupported url scheme %s' % scheme)
-        yield self._proto.connect_df
+        yield self._proto.connect_cb
 
     @_o
     def request(self, url, headers=None, method='GET', body=None):
@@ -96,10 +100,12 @@ class HttpClient(object):
         path = '/' + url.split('/', 3)[3]
 
         if not headers:
-            headers = ordereddict.OrderedDict()
+            headers = OrderedDict()
         headers = self._normalize_headers(headers)
         headers.setdefault('User-Agent', 'monocle/%s' % VERSION)
         headers.setdefault('Host', host)
+        if body is not None:
+            headers.setdefault('Content-Length', str(len(body)))
 
         if not self._proto or not self._proto.transport.connected:
             yield self.connect(host, port, scheme=parts.scheme)
@@ -108,12 +114,15 @@ class HttpClient(object):
         for k, v in headers.iteritems():
             self._proto.sendHeader(k, v)
         self._proto.endHeaders()
-        response = yield self._proto.response_df
+        if body is not None:
+            self._proto.transport.write(body)
+
+        response = yield self._proto.response_cb
 
         self._proto.close()
         self._proto = None
 
-        yield response
+        yield Return(response)
 
 
 class _HttpServerResource(resource.Resource):
@@ -123,7 +132,14 @@ class _HttpServerResource(resource.Resource):
         self.handler = handler
 
     def render(self, request):
-        self.handler(request)
+        @_o
+        def _handler(request):
+            try:
+                yield launch(self.handler, request)
+            except:
+                yield http_respond(request, 500, {},
+                                   "500 Internal Server Error")
+        _handler(request)
         return server.NOT_DONE_YET
 
 
@@ -138,7 +154,7 @@ class HttpServer(object):
 @_o
 def http_respond(request, code, headers, content):
     request.setResponseCode(code)
-    request.setHeader('Server', 'monocle/%s' % VERSION)
+    headers.setdefault('Server', 'monocle/%s' % VERSION)
     for name, value in headers.iteritems():
         request.setHeader(name, value)
     request.write(content)
